/*
Copyright (c) 2004, Stphane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "SHTableView.h"

static NSColor * _gWhiteColor=nil;

@implementation SHTableView

- (NSMenu*) menuForEvent: (NSEvent*)event
{
    NSPoint where;
    int row;

    where = [self convertPoint: [event locationInWindow] fromView: [_window contentView]];
    
    row = [self rowAtPoint: where];
    
    if (row < 0)
    {
        return (showMenuForEmptySelection_==YES) ? [self menu]: nil;
    }
    
    if ([self isRowSelected:row]==NO)
    {
        [self selectRow: row byExtendingSelection: NO];
    }
    
    return [self menu];
}

- (BOOL) acceptsFirstMouse:(NSEvent *)theEvent
{
    return acceptFirstClick_;
}

- (void) setAcceptFirstClick:(BOOL) aBool
{
    acceptFirstClick_=aBool;
}

- (void) showMenuForEmptySelection:(BOOL) aBool
{
    showMenuForEmptySelection_=aBool;
}

- (void)textDidEndEditing:(NSNotification *)notification;
{
    NSMutableDictionary *newUserInfo;
    NSNotification *newNotification;
        
    newUserInfo = [NSMutableDictionary dictionaryWithDictionary:[notification userInfo]];
    [newUserInfo setObject:[NSNumber numberWithInt:0] forKey:@"NSTextMovement"];
    newNotification = [NSNotification notificationWithName:[notification name] object:[notification object] userInfo:newUserInfo];
    [super textDidEndEditing:newNotification];

    // For some reason we lose firstResponder status when when we do the above.
    [[self window] makeFirstResponder:self];
}

- (void)highlightSelectionInClipRect:(NSRect)clipRect
{
    float rowHeight = [self rowHeight] + [self intercellSpacing].height;
    NSRect visibleRect = [self visibleRect];
    NSRect highlightRect;
    NSColor * tRowColor;
    
    if (_gWhiteColor==nil)
    {
        _gWhiteColor=[NSColor whiteColor];
    }
    
    highlightRect.origin = NSMakePoint(NSMinX(visibleRect), (int)(NSMinY(clipRect)/rowHeight)*rowHeight);
    highlightRect.size = NSMakeSize(NSWidth(visibleRect), rowHeight - [self intercellSpacing].height);
    
    if ([[[NSColor colorForControlTint:NSDefaultControlTint] colorNameComponent] isEqualToString:@"graphiteControlTintColor"]==NO)
    {
        tRowColor=[NSColor colorWithCalibratedRed:0.929
                                            green:0.953
                                             blue:1.0
                                            alpha:1];
    }
    else
    {
        tRowColor=[NSColor colorWithDeviceWhite:0.94
                                          alpha:1];
    }
    
    while (NSMinY(highlightRect) < NSMaxY(clipRect))
    {
        NSRect clippedHighlightRect = NSIntersectionRect(highlightRect, clipRect);
        int row = (int)((NSMinY(highlightRect)+rowHeight*0.5)/rowHeight);
        NSColor * tColor = (0 == (row & 0x1)) ? tRowColor : _gWhiteColor;
        
        [tColor set];
        
        NSRectFill(clippedHighlightRect);
        
        highlightRect.origin.y += rowHeight;
    }
    
    [super highlightSelectionInClipRect: clipRect];	// call superclass's behavior
}

- (void) keyDown:(NSEvent *) theEvent
{
    NSString * tString;
    unsigned int stringLength;
    unsigned int i;
    unichar tChar;
    
    tString= [theEvent characters];
    
    stringLength=[tString length];
    
    for(i=0;i<stringLength;i++)
    {
        tChar=[tString characterAtIndex:i];
        
        if (tChar==0x7F)
        {
            NSMenuItem * tMenuItem;
            
            tMenuItem=[[NSMenuItem alloc] initWithTitle:@"" action:@selector(delete:) keyEquivalent:@""];
            
            if ([self validateMenuItem:tMenuItem]==YES)
            {
                [self delete:nil];
            }
            else
            {
                NSBeep();
            }
            
            [tMenuItem release];
            
            return;
        }
    }
    
    [super keyDown:theEvent];
}

- (BOOL)validateMenuItem:(NSMenuItem *)aMenuItem
{
    if ([aMenuItem action]==@selector(delete:))
    {
        if ([self numberOfSelectedRows]>0)
        {
            return YES;
        }
        
        return NO;
    }
    
    return YES;
}

- (IBAction) delete:(id) sender
{
    if ([[self dataSource] respondsToSelector:@selector(deleteSelectedRowsOfTableView:)]==YES)
    {
        [[self dataSource] performSelector:@selector(deleteSelectedRowsOfTableView:)
                                withObject:self];
    }
}

@end
