/*
    Copyright (c) 2001-2003, Stephane Sudre
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification, are permitted
    provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this list of conditions
    and the following disclaimer.

	Redistributions in binary form must reproduce the above copyright notice, this list of conditions
    and the following disclaimer in the documentation and/or other materials provided with the distribution.

	Neither the name of the WhiteBox nor the names of its contributors may be used to endorse 
    or promote products derived from this software without specific prior written permission.


	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
            File Name: SaveHollywoodView.h
             Creation: 05/22/01
    Last modification: 07/01/03
    
    +----------+-------------+---------------------------------------------------------+
    |   Date   |    Author   | Comments                                                |
    +----------+-------------+---------------------------------------------------------+
    | 05/22/01 |   S.Sudre   | Creation                                                |
    +----------+-------------+---------------------------------------------------------+
    | 07/01/03 |   S.Sudre   | Fix: fix a stupid memory leak                           |
    |          |             | Add: Add the BSD License text                           |
    |          |             | Fix: Remove another memory leak                         |
    |          |             | New: Clean the code                                     |
    |          |             | Fix: Remove a flick when the screensaver is launched    |
    |          |             | Update the way the defaults are loaded                  |
    +----------+-------------+---------------------------------------------------------+
    |          |             |                                                         |
    +----------+-------------+---------------------------------------------------------+
*/

#import <AppKit/AppKit.h>
#import <ScreenSaver/ScreenSaver.h>

#import "SHMovieView.h"

#define BORDER_SIZE		50

enum {
    kMovieFrameActualSize,
    kMovieFrameSizeToFit
};

enum {
    kMovieVolumeMute,
    kMovieVolumeNormal,
    kMovieVolumeCustom
};

enum {
    kMovieModeNone,
    kMovieModeFile,
    kMovieModeDirectory,
    kMovieModeMultipleMixed
};

@interface SaveHollywoodView : ScreenSaverView
{
    // Configure Sheet outlets
    
    IBOutlet id	IBconfigureSheet_;
    
    IBOutlet id IBbackgroundColor_;
    IBOutlet id IBdrawBorder_;
    IBOutlet id IBarray_;
    IBOutlet id IBmovieFrame_;
    IBOutlet id IBmovieVolume_;
    IBOutlet id IBrandom_;
    IBOutlet id IBrandomOrder_;
    IBOutlet id IBmainScreen_;
    IBOutlet id IBvolumeSlider_;
    
    IBOutlet id IBpath_;
    IBOutlet id IBdeleteButton_;
    
    IBOutlet id IBversion_;
    
    // Settings attributes
    
    NSColor * backgroundColor_;
    int random_;
    int randomOrder_;
    int drawBorder_;
    int movieFrame_;
    int movieVolume_;
    int mainScreenOnly_;
    
    NSDirectoryEnumerator * enumerator_;
    
    int movieMode_;
    int randomMax_;
    
    float movieCustomVolume_;
    
    NSString * fileLocation_;
    NSMutableArray * fileLocationArray_;
    unsigned int arrayIndex_;
    BOOL inDirectory;
    
    SHMovieView * movieView_;
    
    BOOL preview_;
    BOOL needsSwitching_;
    BOOL mainScreen_;
    
    BOOL brokenPath_;
    
    NSImage * folderIcon_;
    NSImage * movieIcon_;
    
	BOOL loopMode_;
	
	QTMovie * movie_;
	
    NSFileManager * fileManager_;
}

- (IBAction) closeSheet:(id) sender;

- (IBAction) addFile:(id) sender;
- (IBAction) removeFile:(id) sender;

- (IBAction) switchFrameMode:(id) sender;

- (NSString *) getFirstValidInDirectory;
- (NSString *) getNextPath;

- (void) prepareMovieFromURL:(NSURL *) inURL;

- (void) movieDidEndPlaying:(NSNotification *) inNotification;

+ (NSImage *) createIconForFileType:(NSString *) inFileType;
+ (NSImage *) createIconForFile:(NSString *) inPath;

+ (NSString *)colorToString:(NSColor *)theColor;
+ (NSColor *)stringToColor:(NSString *)theString;

- (IBAction) deleteSelectedRowsOfTableView:(NSTableView *) tableView;

@end
