/*
    Copyright (c) 2001-2005, Stephane Sudre
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification, are permitted
    provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this list of conditions
    and the following disclaimer.

	Redistributions in binary form must reproduce the above copyright notice, this list of conditions
    and the following disclaimer in the documentation and/or other materials provided with the distribution.

	Neither the name of the WhiteBox nor the names of its contributors may be used to endorse 
    or promote products derived from this software without specific prior written permission.


	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "SaveHollywoodView.h"
#import <QTKit/QTKit.h>
#import "ImageAndTextCell.h"

#define LRAND()                    ((long) (random() & 0x7fffffff))
#define MAXRAND                    (2147483648.0) /* unsigned 1<<31 as a float */

NSString * const SH_DEFAULTPATH_TIGER=@"/System/Library/CoreServices/Setup Assistant.app/Contents/Resources/TransitionSection.bundle/Contents/Resources/intro.mov";
NSString * const SH_DEFAULTPATH_PANTHER=@"/System/Library/PrivateFrameworks/SetupAssistant.framework/Versions/A/Resources/intro.mov";
NSString * const SH_DEFAULTPATH_JAGUAR=@"/System/Library/PrivateFrameworks/SetupAssistant.framework/Versions/A/Resources/progress.mov";                

int random_no(unsigned int n);

int random_no(unsigned int n)
{
	return ((int) ((n + 1) * (double) LRAND() / MAXRAND));
}

@implementation SaveHollywoodView

- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL) preview
{
    NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
    ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
    
    self=[super initWithFrame:frameRect isPreview:preview];

    if (self!=nil)
    {
        NSString * tString;
        
        fileManager_=[NSFileManager defaultManager];
		
		preview_=preview;
        
        if (preview_==YES)
        {
            mainScreen_=YES;
        }
        else
        {
            mainScreen_= (NSMinX(frameRect)==0 && NSMinY(frameRect)==0);
        }
        
        mainScreenOnly_=[defaults integerForKey:@"MainScreen Only"];
        
        tString=[defaults stringForKey:@"Background Color"];
        
        movieCustomVolume_=1.0;
        
        if (tString==nil)
        {
            backgroundColor_=[[NSColor blackColor] retain];
            random_=NSOffState;
            randomOrder_=NSOffState;
            drawBorder_=NSOnState;
            movieFrame_=kMovieFrameSizeToFit;
            movieVolume_=kMovieVolumeNormal;
            mainScreenOnly_=NSOffState;
        }
        else
        {
            backgroundColor_=[[SaveHollywoodView stringToColor:tString] retain];
            drawBorder_=[defaults integerForKey:@"Draw Border"];
            movieFrame_=[defaults integerForKey:@"Movie Frame"];
            movieVolume_=[defaults integerForKey:@"Movie Volume"];
            
            random_=[defaults integerForKey:@"Random"];
            
            randomOrder_=[defaults integerForKey:@"Random Order"];
            
            if ([defaults objectForKey:@"Movie Custom Volume"]!=nil)
            {
                movieCustomVolume_=[defaults floatForKey:@"Movie Custom Volume"];
            }
        }
        
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            movieView_ = [[SHMovieView alloc] initWithFrame:NSZeroRect];
            
            if (movieView_!=nil)
			{
				[movieView_ setControllerVisible:NO];
				
				[movieView_ setPreservesAspectRatio:NO];
                         
				[self addSubview:movieView_];
			
				if (preview_==NO)
				{
					[[movieView_ window] setInitialFirstResponder:movieView_];
				}
				
				[movieView_ release];
			}
        }
		
		[self setAnimationTimeInterval:0.5];
		
		// Register for notifications
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(movieDidEndPlaying:)
													 name:QTMovieDidEndNotification
												   object:nil];
        
        
        
    }
    
    return self;
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[super dealloc];
}

#pragma mark -

- (NSString *) getFirstValidInDirectory
{
    NSString * tString;
    
    do
    {
        tString = [enumerator_ nextObject];
        
        [enumerator_ skipDescendents];
        
        if (tString!=nil)
        {
            if ([[tString lastPathComponent] characterAtIndex:0]=='.')
            {
                tString=nil;
            }
        }
        else
        {
            break;
        }
    }
    while (tString==nil);
    
    return tString;
}

- (NSString *) getNextPath
{
    int tCount=[fileLocationArray_ count];
    
    if (tCount>0)
    {
        NSString * tFilePath=nil;
        BOOL firstInDirectory_=NO;
        BOOL firstTime=NO;
        NSString * tString;
        BOOL isDirectory;
        
        switch (movieMode_)
        {
            case kMovieModeFile:
                tFilePath=[fileLocationArray_ objectAtIndex:0];
                break;
            case kMovieModeDirectory:
            
                tFilePath=[fileLocationArray_ objectAtIndex:0];
                
                if (enumerator_==nil)
                {
                    [fileManager_ changeCurrentDirectoryPath:tFilePath];
                    
                    enumerator_= [[fileManager_ enumeratorAtPath:tFilePath] retain];
					
                    firstInDirectory_=YES;
                }
                
                tString = [enumerator_ nextObject];
                
                [enumerator_ skipDescendents];
                
                if (tString==nil)
                {
                    if (firstInDirectory_==NO)
                    {
                        [enumerator_ release];
						
                        enumerator_= [[fileManager_ enumeratorAtPath:tFilePath] retain];
                        
                        tString=[self getFirstValidInDirectory];
                    }
                }
                else
                {
                    if ([[tString lastPathComponent] characterAtIndex:0]=='.')
                    {
                        tString=[self getFirstValidInDirectory];
                    }
                }
                
                tFilePath=tString;
                break;
            case kMovieModeMultipleMixed:
                if (arrayIndex_==-1)
                {
                    arrayIndex_=0;
                    firstTime=YES;
                }
                
loop:

                tFilePath=[fileLocationArray_ objectAtIndex:arrayIndex_];
                    
                if ([fileManager_ fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES)
                {
                    tString=nil;
                    
                    if (isDirectory==NO)
                    {
                        if ([[tFilePath lastPathComponent] characterAtIndex:0]=='.')
                        {
                            arrayIndex_++;
							
                            if (arrayIndex_>=tCount)
                            {
                                if (firstTime==YES)
                                {
                                    return nil;
                                }
                                
                                arrayIndex_=0;
                            }
                            
                            goto loop;
                        }
                        else
                        {
                            tString=tFilePath;
							
                            arrayIndex_++;
							
                            if (arrayIndex_>=tCount)
                            {
                                arrayIndex_=0;
                            }
                        }
                    }
                    else
                    {
                        if (enumerator_==nil)
                        {
                            [fileManager_ changeCurrentDirectoryPath:tFilePath];
							
                            enumerator_= [[fileManager_ enumeratorAtPath:tFilePath] retain];
                            
                            firstInDirectory_=YES;
                        }
                    
                        tString = [enumerator_ nextObject];
                    
                        [enumerator_ skipDescendents];
                        
                        if (tString==nil)
                        {
                            if (firstInDirectory_==NO)
                            {
                                [enumerator_ release];
								
                                enumerator_=nil;
                                
                                arrayIndex_++;
								
                                if (arrayIndex_>=tCount)
                                {
                                    if (firstTime==YES)
                                    {
                                        return nil;
                                    }
                                    
                                    arrayIndex_=0;
                                }
                                
                                goto loop;
                            }
                        }
                        else
                        {
                            if ([[tString lastPathComponent] characterAtIndex:0]=='.')
                            {
                                tString=[self getFirstValidInDirectory];
                                
                                if (tString==nil)
                                {
                                    [enumerator_ release];
                                    enumerator_=nil;
                                    
                                    arrayIndex_++;
                                    if (arrayIndex_>=tCount)
                                    {
                                        if (firstTime==YES)
                                        {
                                            return nil;
                                        }
                                        
                                        arrayIndex_=0;
                                    }
                                    
                                    goto loop;
                                }
                            }
                        }
                    }
                }
                else
                {
                    tString=nil;
                    
                    arrayIndex_++;
                    if (arrayIndex_>=tCount)
                    {
                        if (firstTime==YES)
                        {
                            return nil;
                        }
                        
                        arrayIndex_=0;
                    }
                    
                    goto loop;
                }
                
                tFilePath=tString;
                break;
        }
        
        return tFilePath;
    }
    
    return nil;
}

- (void) drawRect:(NSRect) frame
{
    if (mainScreenOnly_!=NSOnState || mainScreen_==YES && brokenPath_==NO)
    {
        [backgroundColor_ set];
    }
    else
    {
        [[NSColor blackColor] set];
    }
	
	NSRectFill(frame);
}

- (void)startAnimation
{
    enumerator_=nil;
    
	loopMode_=NO;
	
    if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
    {
        NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
        ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
        NSArray * tArray;
        BOOL isDirectory;
        NSString * tFilePath;
		
        if (fileLocationArray_!=nil)
        {
            [fileLocationArray_ release];
        
            fileLocationArray_=nil;
        }
        
        tArray=[defaults arrayForKey:@"Array location"];
        
        if (tArray==nil)
        {
            NSString * tArrayString=[defaults stringForKey:@"File location"];
                
            if (tArrayString!=nil && [tArrayString isEqualToString:@""]==NO)
            {
                fileLocationArray_=[[NSMutableArray arrayWithObject:tArrayString] retain];
            }
            else
            {
                if (NSAppKitVersionNumber>=809)
                {
                    // Tiger
                    
                    fileLocationArray_=[[NSMutableArray alloc] initWithObjects:SH_DEFAULTPATH_TIGER,nil];
                }
#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_3
                else
                if (NSAppKitVersionNumber>=663)
                {
                    fileLocationArray_=[[NSMutableArray arrayWithObject:@"/System/Library/PrivateFrameworks/SetupAssistant.framework/Versions/A/Resources/intro.mov"] retain];
                }
                else
                {
                    fileLocationArray_=[[NSMutableArray arrayWithObject:@"/System/Library/PrivateFrameworks/SetupAssistant.framework/Versions/A/Resources/progress.mov"] retain];
                }
#endif
            }
        }
        else
        {
            fileLocationArray_=[[NSMutableArray alloc] initWithCapacity:10];
			
			if (fileLocationArray_!=nil)
			{
				[fileLocationArray_ setArray:tArray];
			}
		}
    
        arrayIndex_=-1;
        
        if ([fileLocationArray_ count]>0)
        {
            tFilePath=[fileLocationArray_ objectAtIndex:0];
            
            if (fileLocationArray_!=nil && [fileManager_ fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES)
            {
                if ([fileLocationArray_ count]==1)
                {        
                    if (isDirectory == YES)
                    {
                        movieMode_=kMovieModeDirectory;
                    }
                    else
                    {
                        movieMode_=kMovieModeFile;
                        
                        if (movieFrame_!=kMovieFrameActualSize || random_!=NSOnState)
                        {
                            loopMode_=YES;
                        }
                    }
                }
                else
                {
                    movieMode_=kMovieModeMultipleMixed;
                }
            }
            else
            {
                movieMode_=kMovieModeNone;
            }
        }
        else
        {
            movieMode_=kMovieModeNone;
        }
       
        
        if (movieMode_!=kMovieModeNone)
        {
            NSURL * tURL;
            
            tFilePath=[self getNextPath];
        
            if (tFilePath==nil)
            {
                brokenPath_=YES;
            }
            else
            {
                if (randomOrder_==NSOnState & movieMode_!=kMovieModeFile)
                {
                    // Count the number of items
                    
                    int i;
                    int tRandom;
                    randomMax_=0;
                    
                    for(i=0;i<[fileLocationArray_ count];i++)
                    {
                        if ([fileManager_ fileExistsAtPath:[fileLocationArray_ objectAtIndex:i] isDirectory:&isDirectory]==YES)
                        {
                            if (isDirectory==NO)
                            {
                                randomMax_++;
                            }
                            else
                            {
                                NSDirectoryEnumerator * tEnumerator;
                                
                                tEnumerator= [fileManager_ enumeratorAtPath:[fileLocationArray_ objectAtIndex:i]];
                                [tEnumerator skipDescendents];
                                
                                randomMax_+=[[tEnumerator allObjects] count];
                            }
                        }
                    }
                    
                    tRandom=random_no(randomMax_-1);
                    
                    for(i=0;i<tRandom;i++)
                    {
                        tFilePath=[self getNextPath];
                    }
                }
                
                tURL=[NSURL fileURLWithPath:tFilePath];
				
				if (tURL!=nil)
				{
					[self prepareMovieFromURL:tURL];
                }
				
                //[[self window] setLevel:[[self window] level]];	// A VOIR SI NECESSAIRE SOUS 10.4
            }
        }
    }
    
    [super startAnimation];
}

- (void) prepareMovieFromURL:(NSURL *) inURL
{
    NSRect tFrame=[self frame];
    float tMovieWidth,tMovieHeight;
	NSError * tError;
	
    movie_=[QTMovie movieWithURL:inURL error:&tError];
        
    if (movie_!=nil)
    {
        // Size and setup the view
        
        NSSize tNaturalSize;
		NSValue * tValue;
		
		// Set Loop Atttribute
		
		[movie_ setAttribute:[NSNumber numberWithBool:loopMode_] forKey:QTMovieLoopsAttribute];
		
		tValue=[movie_ attributeForKey:QTMovieNaturalSizeAttribute];
		
		[tValue getValue:&tNaturalSize];
        
        tMovieWidth=tNaturalSize.width;
        tMovieHeight=tNaturalSize.height;
        
        if (movieFrame_==kMovieFrameActualSize)
        {
            if (tMovieWidth>NSWidth(tFrame) || tMovieHeight>NSHeight(tFrame))
            {
                float h_ratio,v_ratio;
            
                h_ratio=NSWidth(tFrame)/tMovieWidth;
                v_ratio=NSHeight(tFrame)/tMovieHeight;
                
                if (h_ratio>v_ratio) h_ratio=v_ratio;
                
                tMovieWidth*=h_ratio;
                tMovieHeight*=h_ratio;
            }
            else
            {
                if (random_==NSOnState)
                {
                    NSSize tSize;
                    
                    tSize.width=tMovieWidth;
                    tSize.height=tMovieHeight;
                    
                    tFrame.origin=SSRandomPointForSizeWithinRect(tSize,tFrame);
            
                    tFrame.size=tSize;
                    
                    [self lockFocus];
					
                    [backgroundColor_ set];
					
                    NSRectFill([self bounds]);
					
                    [self unlockFocus];
					
					[movieView_ setFrame:tFrame];
					
					[movieView_ setMovie:movie_];
					
                    if (movieVolume_<kMovieVolumeNormal)
                    {
                        [movie_ setMuted:(movieVolume_==kMovieVolumeNormal && mainScreen_==YES) ? NO : YES];
                    }
                    else
                    {
                        [movie_ setVolume:movieCustomVolume_];
                    }
					
					[movieView_ play:self];
            
                    return;
                }
            }
        }
        else
        {
            float h_ratio,v_ratio;
            float inset=0;
            
            if (drawBorder_==NSOnState && preview_==NO)
            {
                inset=2*BORDER_SIZE;
            }
            
            h_ratio=(NSWidth(tFrame)-inset)/tMovieWidth;
            v_ratio=(NSHeight(tFrame)-inset)/tMovieHeight;
            
            if (h_ratio>v_ratio) h_ratio=v_ratio;
            
            tMovieWidth*=h_ratio;
            tMovieHeight*=h_ratio;
        }
    
        // Center the View
            
        tFrame.origin.x=floor((NSWidth(tFrame)-tMovieWidth)*0.5f);
        tFrame.origin.y=floor((NSHeight(tFrame)-tMovieHeight)*0.5f);
        
        tFrame.size.width=tMovieWidth;
        tFrame.size.height=tMovieHeight;
    
        [movieView_ setFrame:tFrame];
		
		[movieView_ setMovie:movie_];
        
        if (movieVolume_<kMovieVolumeNormal)
        {
            [movie_ setMuted:(movieVolume_==kMovieVolumeNormal && mainScreen_==YES) ? NO : YES];
        }
        else
        {
            [movie_ setVolume:movieCustomVolume_];
        }
        
		[self lockFocus];
        
        [self drawRect:[self bounds]];
        
        [self unlockFocus];
		
        [movieView_ play:self];
    }
    else
    {
        
        if (movieMode_==kMovieModeDirectory || movieMode_==kMovieModeMultipleMixed)
        {
            needsSwitching_=YES;
        }
    }
}

- (void)stopAnimation
{
    // Do your animation termination here
    
    if (brokenPath_==NO && movieView_!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            [movieView_ pause:self];
        }
    }
    
    [super stopAnimation];
}

- (BOOL) hasConfigureSheet
{
    return YES;
}

- (NSWindow*)configureSheet
{
    
    if (IBconfigureSheet_ == nil)
    {
        NSTableColumn *tableColumn = nil;
        ImageAndTextCell *imageAndTextCell = nil;
        
        [NSBundle loadNibNamed:@"ConfigureSheet" owner:self];
        
        [IBarray_ registerForDraggedTypes:[NSArray arrayWithObjects:@"SaveHollywood Row Index",NSFilenamesPboardType,nil]];
        
        [IBbackgroundColor_ setColor:backgroundColor_];
        
        [IBrandom_ setEnabled:(movieFrame_==kMovieFrameActualSize) ? YES : NO];
        [IBrandom_ setState:random_];
        [IBrandomOrder_ setState:randomOrder_];
        
        [IBdrawBorder_ setEnabled:(movieFrame_==kMovieFrameSizeToFit) ? YES : NO];
        [IBdrawBorder_ setState:drawBorder_];
        [IBmainScreen_ setState:mainScreenOnly_];
        
        [IBmovieFrame_ selectCellWithTag:movieFrame_];
        [IBmovieVolume_ selectCellWithTag:movieVolume_];
        
        [IBvolumeSlider_ setFloatValue:movieCustomVolume_];
        
        [IBversion_ setStringValue:[[[NSBundle bundleForClass:[self class]] infoDictionary] objectForKey:@"CFBundleVersion"]];
        
        tableColumn = [IBarray_ tableColumnWithIdentifier: @"File Name"];
        imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
        [imageAndTextCell setEditable: YES];
        [tableColumn setDataCell:imageAndTextCell];
        
        folderIcon_=[SaveHollywoodView createIconForFile:@"/bin"];
        movieIcon_=[SaveHollywoodView createIconForFileType:@"mov"];
        
        
        
        [IBarray_ setIntercellSpacing:NSMakeSize(3,1)];
        
    }
    
    [IBdeleteButton_ setEnabled:NO];
    [IBarray_ deselectAll:self];
    
    return IBconfigureSheet_;
}

#pragma mark -

- (BOOL)tableView:(NSTableView *)tv writeRows:(NSArray*)rows 
toPasteboard:(NSPasteboard*)pboard
{
     if ([rows count]==1)
     {
         NSArray * tTypes=[NSArray arrayWithObjects:@"SaveHollywood Row Index",nil];
         int tOriginalRow=[[rows objectAtIndex:0] intValue];

         [pboard declareTypes: tTypes owner:nil];
         [pboard setData: [NSData dataWithBytes: (void *) &tOriginalRow length:sizeof(int)]  forType: @"SaveHollywood Row Index"];

         return YES;
     }
     else
     {
         return NO;
     }
}

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id 
<NSDraggingInfo>)info proposedRow:(int)row 
proposedDropOperation:(NSTableViewDropOperation)op
{
     NSString * tString;
     
     tString=[[info draggingPasteboard] availableTypeFromArray:[NSArray arrayWithObjects:@"SaveHollywood Row Index",NSFilenamesPboardType,nil]];
     
     if (row!=-1 && tString!=nil &&  op==NSTableViewDropAbove)
     {
         if ([tString isEqualToString:@"SaveHollywood Row Index"]==YES)
         {
             int tOriginalRow;

             tOriginalRow= *((int *) [[[info draggingPasteboard] dataForType:@"SaveHollywood Row Index"] bytes]);

             if (row!=tOriginalRow && row!=(tOriginalRow+1))
             {
                 return NSDragOperationMove;
             }
         }
         else
         {
            if ([tString isEqualToString:NSFilenamesPboardType]==YES)
            {
                return NSDragOperationCopy;
            }
         }
     }

     return NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView*)tableView acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation
{
    NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
    ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
    int tOriginalRow;
    NSString * tString;
     
    tString=[[info draggingPasteboard] availableTypeFromArray:[NSArray arrayWithObjects:@"SaveHollywood Row Index",NSFilenamesPboardType,nil]];
     
    if ([tString isEqualToString:@"SaveHollywood Row Index"]==YES)
    {
        tOriginalRow= *((int *) [[[info draggingPasteboard] dataForType:@"SaveHollywood Row Index"] bytes]);
        
        [fileLocationArray_ insertObject:[fileLocationArray_ objectAtIndex:tOriginalRow] atIndex:row];
        
        if (tOriginalRow>row)
        {
            tOriginalRow++;
        }
        
        [fileLocationArray_ removeObjectAtIndex:tOriginalRow];
    }
    else
    if ([tString isEqualToString:NSFilenamesPboardType]==YES)
    {
        NSArray * tArray;
        int i;
        int tCount;
        NSString * nString;
        
        tArray=(NSArray *) [[info draggingPasteboard] propertyListForType:NSFilenamesPboardType];
        
        tCount=[tArray count];
        
        for(i=0;i<tCount;i++)
        {
            nString=[NSString stringWithString:[tArray objectAtIndex:i]];
            
            [fileLocationArray_ insertObject:nString atIndex: (row+i)];
        }
        
    }
    
    [defaults setObject:fileLocationArray_ forKey:@"Array location"];
    
    [defaults synchronize];
    
    [IBarray_ reloadData];
    
    return YES;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    if ([IBarray_ numberOfSelectedRows]==0)
    {
        [IBdeleteButton_ setEnabled:NO];
        [IBpath_ setStringValue:@""];
    }
    else
    {
		[IBdeleteButton_ setEnabled:YES];
        
        if ([IBarray_ numberOfSelectedRows]==1)
        {
			NSDictionary * tAttributeDictionary;
			NSMutableParagraphStyle * tParagrapheStyle;
			NSAttributedString * tAttributedString;
			NSString * tPath;
			
			tPath=[fileLocationArray_ objectAtIndex:[IBarray_ selectedRow]];
			
			tParagrapheStyle=[[NSParagraphStyle defaultParagraphStyle] mutableCopy];
			
			if (tParagrapheStyle!=nil)
			{
				[tParagrapheStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];
			
				tAttributeDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:11.0f],NSFontAttributeName,
																				tParagrapheStyle,NSParagraphStyleAttributeName,
																				nil];
				
				[tParagrapheStyle release];
				
				tAttributedString=[[NSAttributedString alloc] initWithString:tPath attributes:tAttributeDictionary];
				
				if (tAttributedString!=nil)
				{
					[IBpath_ setAttributedStringValue:tAttributedString];
				
					[tAttributedString release];
				}
				else
				{
					[IBpath_ setStringValue:tPath];
				}
			}
		}
        else
        {
            [IBpath_ setStringValue:@""];
        }
    }
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [fileLocationArray_ count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{    
    return [[fileLocationArray_ objectAtIndex:rowIndex] lastPathComponent];
}

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{    
    if ([[tableColumn identifier] isEqualToString: @"File Name"])
    {
        BOOL isDirectory;
       
        [fileManager_ fileExistsAtPath:[fileLocationArray_ objectAtIndex:row] isDirectory:&isDirectory]; 
       
        [(ImageAndTextCell*)cell setImage:(isDirectory==YES) ? folderIcon_ : movieIcon_];
    }
}

#pragma mark -

- (IBAction) deleteSelectedRowsOfTableView:(NSTableView *) tableView
{
    [self removeFile:IBdeleteButton_];
}

- (IBAction)closeSheet:(id)sender
{
    NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
    ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
    
    random_=[IBrandom_ state];
    randomOrder_=[IBrandomOrder_ state];
    drawBorder_=[IBdrawBorder_ state];
    mainScreenOnly_=[IBmainScreen_ state];
    
    movieFrame_= [[IBmovieFrame_ selectedCell] tag];
    movieVolume_=[[IBmovieVolume_ selectedCell] tag];
    
    movieCustomVolume_=[IBvolumeSlider_ floatValue];
    
    [backgroundColor_ release];
    backgroundColor_=[[IBbackgroundColor_ color] retain];
    
    [defaults setObject:[SaveHollywoodView colorToString:backgroundColor_] forKey:@"Background Color"];
    [defaults setInteger:random_ forKey:@"Random"];
    [defaults setInteger:randomOrder_ forKey:@"Random Order"];
    [defaults setInteger:drawBorder_ forKey:@"Draw Border"];
    [defaults setInteger:movieFrame_ forKey:@"Movie Frame"];
    [defaults setInteger:movieVolume_ forKey:@"Movie Volume"];
    [defaults setFloat:movieCustomVolume_ forKey:@"Movie Custom Volume"];
    [defaults setObject:fileLocationArray_ forKey:@"Array location"];
    [defaults setInteger:mainScreenOnly_ forKey:@"MainScreen Only"];
    
    [defaults synchronize];
    
    [NSApp endSheet:IBconfigureSheet_];
}

- (IBAction)addFile:(id)sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    int result;
    
    [oPanel setAllowsMultipleSelection:YES];
    [oPanel setCanChooseDirectories:YES];
    [oPanel setCanChooseFiles:YES];
    [oPanel setPrompt:NSLocalizedStringFromTableInBundle(@"Add",@"Localizable",[NSBundle bundleForClass:[self class]],@"No comment")];
    
    if ((fileLocationArray_==nil) || ([fileLocationArray_ count]==0))
    {
        result = [oPanel runModalForDirectory:nil file:nil types:nil];
    }
    else
    {
        result = [oPanel runModalForDirectory:[fileLocationArray_ objectAtIndex:0] file:nil types:nil];
    }
    
    if (result == NSOKButton)
    {
        NSArray *filesToOpen = [oPanel filenames];
        
        [fileLocationArray_ addObjectsFromArray: filesToOpen];
        
        [IBarray_ reloadData];
    }

}

- (IBAction) removeFile:(id) sender
{
    NSArray * tArray;
    int i,tCount;
    
    tArray=[[IBarray_ selectedRowEnumerator] allObjects];
    
    tCount=[tArray count];
    
    for(i=tCount-1;i>=0;i--)
    {
        [fileLocationArray_  removeObjectAtIndex:[[tArray objectAtIndex:i] intValue]];
    }

    [IBarray_ deselectAll:self];
    [IBarray_ reloadData];
}

- (IBAction) switchFrameMode:(id) sender
{
    [IBdrawBorder_ setEnabled:([[sender selectedCell] tag]==kMovieFrameSizeToFit) ? YES : NO];
    [IBrandom_ setEnabled:([[sender selectedCell] tag]==kMovieFrameActualSize) ? YES : NO];
}

#pragma mark -

- (void) movieDidEndPlaying:(NSNotification *) inNotification
{
	if ([inNotification object]==movie_)
	{
		[self performSelector:@selector(loadNextMovieIfNeeded:) withObject:nil afterDelay:0.0f];
	}
}

- (void) loadNextMovieIfNeeded:(id) sender
{
	if (movieView_!=nil && brokenPath_==NO)
    {
        if ((movieMode_==kMovieModeDirectory || movieMode_==kMovieModeMultipleMixed) || 
			(needsSwitching_==YES) || 
			(movieMode_==kMovieModeFile && movieFrame_==kMovieFrameActualSize && random_==NSOnState))
        {
            NSURL * tURL;
			NSString * tPath=nil;
			int tRandom=1;
			int i;
			
			if (randomOrder_==NSOnState & movieMode_!=kMovieModeFile)
			{
				tRandom=random_no(randomMax_);
				
				if (tRandom<=0)
				{
					tRandom=1;
				}
			}
			
			for(i=0;i<tRandom;i++)
			{
				tPath=[self getNextPath];
			}
			
			if (tPath!=nil)
			{
				tURL=[NSURL fileURLWithPath:tPath];
				
				if (tURL!=nil)
				{
					[self setNeedsDisplay:YES];
					
					[self prepareMovieFromURL:tURL];
				}
			}
            
            needsSwitching_=NO;
        }
    }
}

#pragma mark -

+ (NSString *)colorToString:(NSColor *)theColor
{
    NSColor *tempColor = [theColor colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];

    return([NSString stringWithFormat:@"[%f] [%f] [%f]"
                                   ,[tempColor redComponent]
                                   ,[tempColor greenComponent]
                                   ,[tempColor blueComponent]]);
}

+ (NSColor *)stringToColor:(NSString *)theString
{
    float r,g,b;

    sscanf([theString cString],"[%f] [%f] [%f]",&r,&g,&b);
    return([NSColor colorWithCalibratedRed:r green:g blue:b alpha:1.0]);
}

- (NSSize)windowWillResize:(NSWindow *)sender toSize:(NSSize)proposedFrameSize
{
    NSSize tSize=[sender frame].size;
    
    tSize.height=proposedFrameSize.height;
    
    return tSize;
}

+ (NSImage *) createIconForFileType:(NSString *) inFileType
{
    NSSize tSize;
    NSImage * tImage;
    NSImage * nIcon;
    
	tImage=[[NSWorkspace sharedWorkspace] iconForFileType:inFileType];
        
	tSize=[tImage size];
        
    nIcon=[[NSImage alloc] initWithSize:NSMakeSize(16,16)];
        
    [nIcon lockFocus];
        
    [tImage drawInRect:NSMakeRect(0,0,16,16)
              fromRect:NSMakeRect(0,0,tSize.width,tSize.height)
             operation:NSCompositeSourceOver
              fraction:1.0];
        
    [nIcon unlockFocus];
    
    return nIcon;
}

+ (NSImage *) createIconForFile:(NSString *) inPath
{
    NSSize tSize;
    NSImage * tImage;
    NSImage * nIcon;
    
	tImage=[[NSWorkspace sharedWorkspace] iconForFile:inPath];
        
	tSize=[tImage size];
        
    nIcon=[[NSImage alloc] initWithSize:NSMakeSize(16,16)];
        
    [nIcon lockFocus];
        
    [tImage drawInRect:NSMakeRect(0,0,16,16)
              fromRect:NSMakeRect(0,0,tSize.width,tSize.height)
             operation:NSCompositeSourceOver
              fraction:1.0];
        
    [nIcon unlockFocus];
    
    return nIcon;
}

@end
